@@ -49,7 +49,7 @@ my @packages;    # list of packages to compile. 5.6 only
 sub nice ($) { }
 my $PERL510 = ( $] >= 5.009005 );
 my $PERL511 = ( $] >= 5.011 );
-my $PERL56  = ( $] <  5.008 );
+my $PERL56  = ( $] <  5.008001 );
 
 my %optype_enum;
 my ($SVt_PVGV, $SVf_FAKE, $POK);
@@ -164,7 +164,7 @@ sub B::SV::ix {
     #nice "\tsvtab ".$$sv." => bsave(".$ix.");
     $sv->bsave($ix);
     $ix;
-    }
+  }
 }
 
 sub B::GV::ix {
@@ -303,7 +303,11 @@ sub B::NULL::bsave {
 
   nice '-' . class($sv) . '-', asm "ldsv", $varix = $ix, sv_flags($sv)
     unless $ix == $varix;
-  asm "sv_refcnt", $sv->REFCNT;
+  if ($PERL56) {
+    asm "stsv", $ix;
+  } else {
+    asm "sv_refcnt", $sv->REFCNT;
+  }
 }
 
 sub B::SV::bsave;
@@ -442,7 +446,7 @@ sub B::PVLV::bsave {
 sub B::BM::bsave {
   my ( $sv, $ix ) = @_;
   $sv->B::PVMG::bsave($ix);
-  asm "xpv_cur",      $sv->CUR;
+  asm "xpv_cur",      $sv->CUR if $] > 5.008;
   asm "xbm_useful",   $sv->USEFUL;
   asm "xbm_previous", $sv->PREVIOUS;
   asm "xbm_rare",     $sv->RARE;
@@ -511,8 +515,6 @@ sub B::AV::bsave {
 
   nice "-AV-", asm "ldsv", $varix = $ix, sv_flags($av) unless $ix == $varix;
   asm "av_extend", $av->MAX if $av->MAX >= 0;
-
-  # asm "av_pushx", $_->ix, sv_flags($_) for @array;
   asm "av_pushx", $_ for @array;
   asm "sv_refcnt", $av->REFCNT;
   if ( !$PERL510 ) {        # VERSION < 5.009
@@ -717,10 +719,8 @@ sub B::PMOP::bsave {
       $rstart = $op->pmreplstart->ix;
     }
     elsif ( $op->name eq 'pushre' ) {
-      $rrop  = "op_pmreplrootpo";
       $rrarg = $op->pmreplroot;
-
-      # 5.9 $op->pmtargetoff?
+      $rrop  = "op_pmreplrootpo";
     }
     $op->B::BINOP::bsave($ix);
     if ( $op->pmstashpv )
@@ -729,7 +729,6 @@ sub B::PMOP::bsave {
         asm "op_pmstashpv", pvix $op->pmstashpv;
       }
       else {
-
         # crash in 5.10, 5.11
         bwarn("op_pmstashpv ignored") if $debug{M};
       }
@@ -740,9 +739,11 @@ sub B::PMOP::bsave {
     }
   }
   else {
-    $rrop   = "op_pmreplrootgv";
+    $rrop  = "op_pmreplrootgv";
     $rrarg  = $op->pmreplroot->ix;
     $rstart = $op->pmreplstart->ix if $op->name eq 'subst';
+    # 5.6 walks down the pmreplrootgv here
+    # $op->pmreplroot->save($rrarg) unless $op->name eq 'pushre';
     my $stashix = $op->pmstash->ix unless $PERL56;
     $op->B::BINOP::bsave($ix);
     asm "op_pmstash", $stashix unless $PERL56;
@@ -777,7 +778,6 @@ sub B::PMOP::bsave {
       bwarn( "PMOP pmstashpv: ", $op->pmstashpv, ", pmflags: ", $op->pmflags )
         if $debug{M};
     }
-    #asm "op_reflags", $op->reflags;
     asm "newpv", pvstring $op->precomp;
     asm "pregcomp";
     # pregcomp does not set the extflags correctly, just the pmflags
@@ -861,7 +861,7 @@ sub B::OP::opwalk {
   my $ix = $optab{$$op};
   defined($ix) ? $ix : do {
     my $ix;
-    my @oplist = $op->oplist; # FIXME for 5.6. called by a COP there
+    my @oplist = ($PERL56 and $op->isa("B::COP")) ? () : $op->oplist; # 5.6 may be called by a COP
     push @cloop, undef;
     $ix = $_->ix while $_ = pop @oplist;
     while ( $_ = pop @cloop ) {
@@ -869,7 +869,7 @@ sub B::OP::opwalk {
       asm "op_next", $optab{ ${ $_->next } };
     }
     $ix;
-    }
+  }
 }
 
 sub save_cq {
@@ -1089,7 +1089,7 @@ use ByteLoader '$ByteLoader::VERSION';
       my $dh = $PERL56 ? *main::DATA : *{ defstash->NAME . "::DATA" };
       unless ( eof $dh ) {
         local undef $/;
-        asm "data", ord 'D';
+        asm "data", ord 'D' if !$PERL56;
         print <$dh>;
       }
       else {
